/***************************************************************************
                          dlgconfiguration.cpp  -  description
                             -------------------
    begin                : Tue Jul 10 2001
    copyright            : (C) 2001, 2002, 2003 by The KXMLEditor Team
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgconfiguration.h"


#include <qlayout.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qcombobox.h>

#include <klocale.h>
#include <kiconloader.h>
#include <kconfig.h>
#include <kfontcombo.h>
#include <knuminput.h>

#define CONFIG_DEFAULT_VERSION "Default version"
#define CONFIG_DEFAULT_ENCODING "Default encoding"
#define CONFIG_DEFAULT_BEHAVIOUR "Default behaviour"

#define CONFIG_PRINT_FONTFAMILY "Print font family"
#define CONFIG_PRINT_FONTSIZE "Print font size"
#define CONFIG_PRINT_INDENT "Print indentation"
#define CONFIG_PRINT_HAS_FOOTER "Print has footer"
#define CONFIG_PRINT_HAS_HEADER "Print has header"

#define CONFIG_TEXT_EDITOR_COLOR_DEFAULT_TEXT "DefaultText"
#define CONFIG_TEXT_EDITOR_COLOR_ELEMENT_NAMES "ElementNames"
#define CONFIG_TEXT_EDITOR_COLOR_ATTR_NAMES "AttributeNames"
#define CONFIG_TEXT_EDITOR_COLOR_ATTR_VALUES "AttributeValues"
#define CONFIG_TEXT_EDITOR_COLOR_SYNTAX_CHARS "SyntaxChars"
#define CONFIG_TEXT_EDITOR_COLOR_COMENTS "Comments"
#define CONFIG_TEXT_EDITOR_COLOR_ERRORS "SyntaxError"

KXEConfigurationDialog::KXEConfigurationDialog( QWidget * pParent, const char * pszName, bool bModal )
	: KDialogBase( IconList,
	               i18n("Configure"),
	               Apply | Ok | Cancel | Help,
	               Ok,
	               pParent,
	               pszName,
	               bModal,
	               true )
{
	QFrame * pTmpFrame;
	QVBoxLayout * pTmpLayout;

  // setting up treeview page
	pTmpFrame = addPage( i18n("Tree View"),
	                     i18n("Tree View properties"),
	                     KGlobal::instance()->iconLoader()->loadIcon("view_tree", KIcon::NoGroup, KIcon::SizeMedium) );
	m_pPageTreeView = new ConfPage_TreeView( pTmpFrame );
	pTmpLayout = new QVBoxLayout( pTmpFrame, 0, spacingHint() );
	pTmpLayout->addWidget( m_pPageTreeView );
	m_pPageTreeView->m_pCheckBoxEnableDragDrop->setEnabled(false); // TODO remove, when a drag&drop is implemented and should be used

	//setting up default version & encoding page
	pTmpFrame = addPage( i18n("Defaults"),
	                     i18n("Default file settings"),
	                     KGlobal::instance()->iconLoader()->loadIcon("files", KIcon::NoGroup, KIcon::SizeMedium) );
	m_pPageDefaults = new ConfPage_Defaults( pTmpFrame );
	pTmpLayout = new QVBoxLayout( pTmpFrame, 0, spacingHint() );
	pTmpLayout->addWidget( m_pPageDefaults );
	fillEncodings(m_pPageDefaults->m_pComboBoxEncoding);

	//setting up printout settings page
	pTmpFrame = addPage( i18n("Printout"),
	                     i18n("Printout settings"),
	                     KGlobal::instance()->iconLoader()->loadIcon("printer2", KIcon::NoGroup, KIcon::SizeMedium) );
	m_pPagePrint = new ConfPage_Print( pTmpFrame );
	pTmpLayout = new QVBoxLayout( pTmpFrame, 0, spacingHint() );
	pTmpLayout->addWidget( m_pPagePrint );

	// setting up tar.gz-extensions page
	pTmpFrame = addPage( i18n("TarGz Extensions"),
	                     i18n("Define TarGz file extensions"),
	                     KGlobal::instance()->iconLoader()->loadIcon("filetypes", KIcon::NoGroup, KIcon::SizeMedium) );
	m_pPageTarGzExt = new ConfPage_TarGzExt( pTmpFrame );
	pTmpLayout = new QVBoxLayout( pTmpFrame, 0, spacingHint() );
	pTmpLayout->addWidget( m_pPageTarGzExt );
	connect( m_pPageTarGzExt->m_pListView, SIGNAL(selectionChanged()), this, SLOT(slotListTarGzExtSelChanged()) );
	connect( m_pPageTarGzExt->m_pEditExtension, SIGNAL(textChanged(const QString&)), this, SLOT(slotExtensionChanged(const QString&)) );
	connect( m_pPageTarGzExt->m_pBtnNew, SIGNAL(clicked()), this, SLOT(slotNewExtension()) );
	connect( m_pPageTarGzExt->m_pBtnDelete, SIGNAL(clicked()), this, SLOT(slotDeleteExtension()) );

  // setting up text editor page
	pTmpFrame = addPage( i18n("Text editor"),
	                     i18n("Text editor properties"),
	                     KGlobal::instance()->iconLoader()->loadIcon("colorize", KIcon::NoGroup, KIcon::SizeMedium) );
	m_pPageTextEditor = new ConfPage_TextEditor( pTmpFrame );
	pTmpLayout = new QVBoxLayout( pTmpFrame, 0, spacingHint() );
	pTmpLayout->addWidget( m_pPageTextEditor );
}


void KXEConfigurationDialog::slotExtensionChanged( const QString & szText )
{
	if ( m_pPageTarGzExt->m_pListView->selectedItem() )
		m_pPageTarGzExt->m_pListView->selectedItem()->setText( 0, szText );
}

void KXEConfigurationDialog::slotNewExtension()
{
	QListViewItem * pItem = new QListViewItem( m_pPageTarGzExt->m_pListView, "" );
	m_pPageTarGzExt->m_pListView->setCurrentItem( pItem );
	m_pPageTarGzExt->m_pListView->setSelected( pItem, true );

	if ( m_pPageTarGzExt->m_pListView->selectedItem() )
	{
		m_pPageTarGzExt->m_pEditExtension->setEnabled(true);
		m_pPageTarGzExt->m_pLabelExtension->setEnabled(true);
		m_pPageTarGzExt->m_pEditExtension->setFocus();
	}
}

void KXEConfigurationDialog::slotDeleteExtension()
{
	QListViewItem * pDelItem = m_pPageTarGzExt->m_pListView->selectedItem();
	if ( pDelItem )
	{
		QListViewItem * pNextItem = pDelItem->itemAbove();
		if ( ! pNextItem )
			pNextItem = pDelItem->itemBelow();

		m_pPageTarGzExt->m_pEditExtension->clear();
		m_pPageTarGzExt->m_pEditExtension->setEnabled(false);
		m_pPageTarGzExt->m_pLabelExtension->setEnabled(false);

		m_pPageTarGzExt->m_pListView->takeItem( pDelItem );
		delete pDelItem;

		if ( pNextItem )
			m_pPageTarGzExt->m_pListView->setSelected( pNextItem, true );
	}
}

void KXEConfigurationDialog::slotListTarGzExtSelChanged()
{
	if ( m_pPageTarGzExt->m_pListView->selectedItem() )
	{
		m_pPageTarGzExt->m_pEditExtension->setText( m_pPageTarGzExt->m_pListView->selectedItem()->text(0) );
		m_pPageTarGzExt->m_pListView->setEnabled(true);
		m_pPageTarGzExt->m_pLabelExtension->setEnabled(true);
		m_pPageTarGzExt->m_pEditExtension->setEnabled(true);
	}
}



void KXEConfigurationDialog::slotOk()
{
	slotApply();
	accept();
}

void KXEConfigurationDialog::slotApply()
{
	emit configurationChanged( this );
}

void KXEConfigurationDialog::readConfiguration( KConfig * pConfig )
{
	pConfig->setGroup("Tree View");
	m_pPageTreeView->m_pSpinBoxExpandLevel->setValue( pConfig->readNumEntry("Expand to level", 5) );
	m_pPageTreeView->m_pCheckBoxRootDecorated->setChecked( pConfig->readBoolEntry("Root decorated", false) );
	m_pPageTreeView->m_pCheckBoxEnableDragDrop->setChecked( pConfig->readBoolEntry("Enable DnD", true) );
	switch (pConfig->readNumEntry("Element display mode", 0))
	{
		case 0:
			m_pPageTreeView->m_pBtnEDM_NoAttributes->setChecked(true);
			break;
		case 1:
			m_pPageTreeView->m_pBtnEDM_NamesOnly->setChecked(true);
			break;
		case 2:
			m_pPageTreeView->m_pBtnEDM_NamesValues->setChecked(true);
			break;
		default:
			m_pPageTreeView->m_pBtnEDM_NoAttributes->setChecked(true);
	}

	pConfig->setGroup("TarGz Extensions");
	QStrList strListOfTarGzExtensions;
	pConfig->readListEntry("Extensions", strListOfTarGzExtensions);
	QString szExtension;
	for ( szExtension = strListOfTarGzExtensions.first();
	      szExtension != 0;
	      szExtension = strListOfTarGzExtensions.next() )
		static_cast <void> ( new QListViewItem( m_pPageTarGzExt->m_pListView, szExtension ) );

	// configuration for defaults page
	m_pPageDefaults->m_pLineEditVersion->setText(pConfig->readEntry(CONFIG_DEFAULT_VERSION,"1.0"));
	m_pPageDefaults->m_pComboBoxEncoding->setCurrentText(pConfig->readEntry(CONFIG_DEFAULT_ENCODING,"ISO-8859-1"));
	setDefaultBehaviour((NewFileBehaviour)pConfig->readNumEntry(CONFIG_DEFAULT_BEHAVIOUR,1));

	// configuration for printouts
  pConfig->setGroup("Printing");
	m_pPagePrint->m_pFontFamily->setCurrentText(pConfig->readEntry(CONFIG_PRINT_FONTFAMILY,"Courier"));
	m_pPagePrint->m_pFontSize->setValue(pConfig->readNumEntry(CONFIG_PRINT_FONTSIZE,10));
	m_pPagePrint->m_pIndentation->setValue(pConfig->readNumEntry(CONFIG_PRINT_INDENT,2));
	m_pPagePrint->m_pPrintFooter->setChecked(pConfig->readBoolEntry(CONFIG_PRINT_HAS_FOOTER,true));
	m_pPagePrint->m_pPrintHeader->setChecked(pConfig->readBoolEntry(CONFIG_PRINT_HAS_HEADER,true));

  pConfig->setGroup("Text Editor");
  m_pPageTextEditor->m_pColorButtonDefaultText->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_DEFAULT_TEXT, "#000000")));
  m_pPageTextEditor->m_pColorButtonElementNames->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_ELEMENT_NAMES, "#800000"))); 
  m_pPageTextEditor->m_pColorButtonAttributeNames->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_ATTR_NAMES, "#00ffff"))); 
  m_pPageTextEditor->m_pColorButtonAttributeValues->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_ATTR_VALUES, "#00ff00"))); 
  m_pPageTextEditor->m_pColorButtonXmlSyntaxChars->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_SYNTAX_CHARS, "#000080"))); 
  m_pPageTextEditor->m_pColorButtonComments->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_COMENTS, "#808080"))); 
  m_pPageTextEditor->m_pColorButtonSyntaxError->setColor(QColor(pConfig->readEntry(CONFIG_TEXT_EDITOR_COLOR_ERRORS, "#ff0000")));
}

void KXEConfigurationDialog::saveConfiguration(KConfig * pConfig ) const
{
	pConfig->setGroup("Tree View");
	pConfig->writeEntry("Root decorated", getTreeViewRootDecorated() );
	pConfig->writeEntry("Expand to level", getTreeViewExpandToLevel() );
	pConfig->writeEntry("Enable DnD", getTreeViewEnableDragDrop() );
	pConfig->writeEntry("Element display mode", getTreeViewElemDisplayMode() );

	pConfig->setGroup("TarGz Extensions");
	pConfig->writeEntry("Extensions", getListOfTarGzExtensions() );

  pConfig->setGroup("Printing");
	pConfig->writeEntry(CONFIG_DEFAULT_VERSION,getDefaultVersion());
	pConfig->writeEntry(CONFIG_DEFAULT_ENCODING,getDefaultEncoding());
	pConfig->writeEntry(CONFIG_DEFAULT_BEHAVIOUR,(int)getDefaultBehaviour());

	pConfig->writeEntry(CONFIG_PRINT_FONTFAMILY,getPrintFontFamily());
	pConfig->writeEntry(CONFIG_PRINT_FONTSIZE,(int)getPrintFontSize());
	pConfig->writeEntry(CONFIG_PRINT_INDENT,(int)getPrintIndentation());
	pConfig->writeEntry(CONFIG_PRINT_HAS_FOOTER,(bool)printFooter());
	pConfig->writeEntry(CONFIG_PRINT_HAS_HEADER,(bool)printHeader());

  pConfig->setGroup("Text Editor");
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_DEFAULT_TEXT, colorDefaultText().name());
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_ELEMENT_NAMES, colorElementNames().name());
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_ATTR_NAMES, colorAttributeNames().name());
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_ATTR_VALUES, colorAttributeValues().name());
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_SYNTAX_CHARS, colorXmlSyntaxChars().name());
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_COMENTS, colorComments().name());
  pConfig->writeEntry(CONFIG_TEXT_EDITOR_COLOR_ERRORS, colorSyntaxError().name());
  
	pConfig->sync();
}

int KXEConfigurationDialog::getTreeViewElemDisplayMode() const
{
	if ( m_pPageTreeView->m_pBtnEDM_NoAttributes->isChecked() )
		return 0;
	else if ( m_pPageTreeView->m_pBtnEDM_NamesOnly->isChecked() )
		return 1;
	else if ( m_pPageTreeView->m_pBtnEDM_NamesValues->isChecked() )
		return 2;
	else
		return 0;
}

QStrList KXEConfigurationDialog::getListOfTarGzExtensions() const
{
        QStrList list;

        QListViewItem * pTarGzItem = m_pPageTarGzExt->m_pListView->firstChild();
        while (pTarGzItem)
        {
                list.append( pTarGzItem->text(0) );
                pTarGzItem = pTarGzItem->nextSibling();
        }

        return list;
}

QString KXEConfigurationDialog::getDefaultVersion() const
{
    return m_pPageDefaults->m_pLineEditVersion->text();
}

QString KXEConfigurationDialog::getDefaultEncoding() const
{
    return m_pPageDefaults->m_pComboBoxEncoding->currentText();
}

void KXEConfigurationDialog::fillEncodings(QComboBox *pComboBox)
{
	pComboBox->insertItem("UTF-8");
	pComboBox->insertItem("ISO-8859-1");
	pComboBox->insertItem("ISO-8859-2");
	pComboBox->insertItem("ISO-8859-3");
	pComboBox->insertItem("ISO-8859-4");
	pComboBox->insertItem("ISO-8859-5");
	pComboBox->insertItem("ISO-8859-6");
	pComboBox->insertItem("ISO-8859-7");
	pComboBox->insertItem("ISO-8859-8");
	pComboBox->insertItem("ISO-8859-9");
}

KXEConfigurationDialog::NewFileBehaviour KXEConfigurationDialog::getDefaultBehaviour() const
{
    if (m_pPageDefaults->m_pNewEmptyFile->isChecked())
    	return CreateEmptyFile;

    else if (m_pPageDefaults->m_pStartWithAssistance->isChecked())
    	return CreateWithAssistance;

    return UseDefaults;
}

void KXEConfigurationDialog::setDefaultBehaviour(KXEConfigurationDialog::NewFileBehaviour behaviour)
{
	m_pPageDefaults->m_pNewEmptyFile->setChecked(behaviour == CreateEmptyFile);
	m_pPageDefaults->m_pStartWithAssistance->setChecked(behaviour == CreateWithAssistance);
	m_pPageDefaults->m_pUseDefaultSettings->setChecked(behaviour == UseDefaults);
}

QString KXEConfigurationDialog::getPrintFontFamily() const
{
	return m_pPagePrint->m_pFontFamily->currentText();
}

bool KXEConfigurationDialog::printHeader() const
{
	return m_pPagePrint->m_pPrintHeader->isChecked();
}

uint KXEConfigurationDialog::getPrintFontSize() const
{
	return m_pPagePrint->m_pFontSize->value();
}

uint KXEConfigurationDialog::getPrintIndentation() const
{
	return m_pPagePrint->m_pIndentation->value();
}

bool KXEConfigurationDialog::printFooter() const
{
	return m_pPagePrint->m_pPrintFooter->isChecked();
}
