/***************************************************************************
                          dlgconfiguration.h  -  description
                             -------------------
    begin                : Tue Jul 10 2001
    copyright            : (C) 2001, 2002, 2003 by The KXMLEditor Team
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DLGCONFIGURATION_H
#define DLGCONFIGURATION_H

#include <kdialogbase.h>

#include "confpage_treeview.h"
#include "confpage_targzext.h"
#include "confpage_defaults.h"
#include "confpage_print.h"
#include "confpage_text_editor.h"

#include <qstringlist.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qspinbox.h>

#include <kcolorbutton.h>

class KConfig;


/**
 * Applications configuration dialog.
 * @author Olaf Hartig
 */
class KXEConfigurationDialog : public KDialogBase
{
		Q_OBJECT

	public:

	enum NewFileBehaviour
	{
		CreateEmptyFile,
		CreateWithAssistance,
		UseDefaults
	};

		KXEConfigurationDialog( QWidget * pParent, const char * pszName = 0, bool bModal = true );

		/** Reads the configuration from the given config file. */
		void readConfiguration( KConfig * );

		/** Saves the current configuration to the given config file. */
		void saveConfiguration( KConfig * ) const;

		bool getTreeViewRootDecorated() const { return m_pPageTreeView->m_pCheckBoxRootDecorated->isChecked(); }
		unsigned int getTreeViewExpandToLevel() const { return m_pPageTreeView->m_pSpinBoxExpandLevel->value(); }
		bool getTreeViewEnableDragDrop() const { return m_pPageTreeView->m_pCheckBoxEnableDragDrop->isChecked(); }
		int getTreeViewElemDisplayMode() const;

		QStrList getListOfTarGzExtensions() const;
		/** Returns default XML version which is going to be proposed in file->new dialog. */
		QString getDefaultVersion() const;
		/** Returns default XML encoding which is going to be proposed in file->new dialog. */
		QString getDefaultEncoding() const;
		/** Fills combobox with set of available encodings */
		static void fillEncodings(QComboBox *pComboBox);
		/** Returns required behaviour when new file have to be created */
		KXEConfigurationDialog::NewFileBehaviour getDefaultBehaviour() const;
		/** Sets default behaviour when new file is about to be created */
		void setDefaultBehaviour(KXEConfigurationDialog::NewFileBehaviour behaviour);
		/** Returns font type used for printouts. */
		QString getPrintFontFamily() const;
		/** Returns true if printouts should have header. */
		bool printHeader() const;
		/** Returns size of the font used for printouts. @sa getPrintFontFamily() */
		uint getPrintFontSize() const;
		/** Returns size of XML element indentation (counted in space
		characters) for printouts. */
		uint getPrintIndentation() const;
		/** Returns true if printouts should have footer. */
		bool printFooter() const;

    // get syntax highlighting configuration
    QColor colorDefaultText() const { return m_pPageTextEditor->m_pColorButtonDefaultText->color(); }
    QColor colorElementNames() const { return  m_pPageTextEditor->m_pColorButtonElementNames->color(); }
    QColor colorAttributeNames() const { return  m_pPageTextEditor->m_pColorButtonAttributeNames->color(); }
    QColor colorAttributeValues() const { return  m_pPageTextEditor->m_pColorButtonAttributeValues->color(); }
    QColor colorXmlSyntaxChars() const { return  m_pPageTextEditor->m_pColorButtonXmlSyntaxChars->color(); }
    QColor colorComments() const { return  m_pPageTextEditor->m_pColorButtonComments->color(); }
    QColor colorSyntaxError() const { return  m_pPageTextEditor->m_pColorButtonSyntaxError->color(); }    

protected:

		ConfPage_TreeView * m_pPageTreeView;
		ConfPage_TarGzExt * m_pPageTarGzExt;
		ConfPage_Defaults *m_pPageDefaults;
		ConfPage_Print* m_pPagePrint;
    ConfPage_TextEditor *m_pPageTextEditor;

protected slots:

		/** Called when user presses Apply button */
		void slotApply();

		/** Called when user presses OK button */
		void slotOk();

		/** Called, when Delete button clicked */
		void slotDeleteExtension();

		/** Called, when New button clicked */
		void slotNewExtension();

		/** Called, when text in edit line changed  */
		void slotExtensionChanged( const QString & );

		/** Called, when user select item in list of extensions */
		void slotListTarGzExtSelChanged();

signals:

		/** Emited, when user press Apply or Ok buttons */
		void configurationChanged( const KXEConfigurationDialog * const pNewConfiguration );
};

#endif
